@@ -27,6 +27,7 @@ executors adhere to task status and do not build tasks if they are marked
 as Aborted or Skipped.
 
 """
+import os
 from collections.abc import Iterable
 import traceback
 from copy import copy, deepcopy
@@ -36,6 +37,7 @@ import logging
 import collections
 import tempfile
 from math import ceil
+from functools import partial
 
 try:
     import importlib.resources as importlib_resources
@@ -52,11 +54,13 @@ from IPython.display import Image
 from ploomber.Table import Table, TaskReport, BuildReport
 from ploomber.products import MetaProduct
 from ploomber.util import (image_bytes2html, isiterable, requires, markup)
+from ploomber.util.debug import debug_if_exception
 from ploomber import resources
 from ploomber import executors
 from ploomber.constants import TaskStatus, DAGStatus
 from ploomber.exceptions import (DAGBuildError, DAGRenderError,
                                  DAGBuildEarlyStop)
+from ploomber.executors import Serial
 from ploomber.MessageCollector import MessageCollector
 from ploomber.util.util import callback_check
 from ploomber.dag.DAGConfiguration import DAGConfiguration
@@ -293,12 +297,14 @@ class DAG(collections.abc.Mapping):
 
             if show_progress:
                 tasks = tqdm(self.values(), total=len(self))
+            else:
+                tasks = self.values()
 
             exceptions = MessageCollector()
             warnings_ = MessageCollector()
 
             # reset all tasks status
-            for task in self.values():
+            for task in tasks:
                 task.exec_status = TaskStatus.WaitingRender
 
             for t in tasks:
@@ -341,21 +347,36 @@ class DAG(collections.abc.Mapping):
 
         self._exec_status = DAGStatus.WaitingExecution
 
-    def build(self, force=False, show_progress=True):
+    def build(self, force=False, show_progress=True, debug=False):
         """
         Runs the DAG in order so that all upstream dependencies are run for
         every task
 
         Parameters
         ----------
-        force: bool, optional
+        force : bool, default=False
             If True, it will run all tasks regardless of status, defaults to
             False
 
+        show_progress : bool, default=True
+            Show progress bar
+
+        debug : bool, default=False
+            Drop a debugging session if building raises an exception. Note that
+            this modifies the executor and temporarily sets it to Serial
+            with subprocess off and catching exceptions/warnings off. Restores
+            the original executor at the end.
+
         Notes
         -----
         All dag-level clients are closed after calling this function
 
+        ``debug`` is useful to let a pipeline run and start debugging at a
+        failing  PythonCallable task but it won't work with failing
+        ``NotebookRunner`` tasks because notebooks/scripts are executed in a
+        different process. If you want to debug ``NotebookRunner`` tasks, use
+        ``NotebookRunner.debug()`` instead.
+
         Returns
         -------
         BuildReport
@@ -372,8 +393,30 @@ class DAG(collections.abc.Mapping):
         else:
             dag_logger = DAGLogger(handler=res)
 
+        # if debug, we have to change the executor to these settings, if we run
+        # tasks in a subprocess or catch exception, we won't be able to start
+        # the debugging session in the right place
+        if debug:
+            executor_original = self.executor
+            self.executor = Serial(build_in_subprocess=False,
+                                   catch_exceptions=False,
+                                   catch_warnings=False)
+
+        callable_ = partial(self._build,
+                            force=force,
+                            show_progress=show_progress)
+
         with dag_logger:
-            report = self._build(force, show_progress)
+            try:
+                if debug:
+                    report = debug_if_exception(callable_)
+                else:
+                    report = callable_()
+            finally:
+                self._close_clients()
+
+        if debug:
+            self.executor = executor_original
 
         return report
 
@@ -470,6 +513,21 @@ class DAG(collections.abc.Mapping):
                 raise DAGBuildError(
                     'Failed to build DAG {}'.format(self)) from build_exception
 
+    def _close_clients(self):
+        """Close all clients (dag-level, task-level and product-level)
+        """
+        for client in self.clients.values():
+            client.close()
+
+        for task_name in self._iter():
+            task = self[task_name]
+
+            if task.client:
+                task.client.close()
+
+            if task.product.client:
+                task.product.client.close()
+
     def _run_on_failure(self, tb):
         if self.on_failure:
             self._logger.debug('Executing on_failure hook '
@@ -495,8 +553,30 @@ class DAG(collections.abc.Mapping):
             self._logger.debug('No on_finish hook for dag '
                                '"%s", skipping', self.name)
 
-    def build_partially(self, target, force=False, show_progress=True):
+    def build_partially(self,
+                        target,
+                        force=False,
+                        show_progress=True,
+                        debug=False):
         """Partially build a dag until certain task
+
+        Parameters
+        ----------
+        target : str,
+            Name of the target task (last one to build)
+
+        force : bool, default=False
+            If True, it will run all tasks regardless of status, defaults to
+            False
+
+        show_progress : bool, default=True
+            Show progress bar
+
+        debug : bool, default=False
+            Drop a debugging session if building raises an exception. Note that
+            this modifies the executor and temporarily sets it to Serial
+            with subprocess off and catching exceptions/warnings off. Restores
+            the original executor at the end.
         """
         lineage = self[target]._lineage
         dag_copy = deepcopy(self)
@@ -513,7 +593,9 @@ class DAG(collections.abc.Mapping):
         # will make it outdated, we have to force reload from disk
         self._clear_metadata()
 
-        return dag_copy.build(force=force, show_progress=show_progress)
+        return dag_copy.build(force=force,
+                              show_progress=show_progress,
+                              debug=debug)
 
     def status(self, **kwargs):
         """Returns a table with tasks status
@@ -559,7 +641,8 @@ class DAG(collections.abc.Mapping):
             status = False
 
         if 'plot' in sections:
-            _, path_to_plot = tempfile.mkstemp(suffix='.png')
+            fd, path_to_plot = tempfile.mkstemp(suffix='.png')
+            os.close(fd)
             self.plot(output=path_to_plot)
             plot = image_bytes2html(Path(path_to_plot).read_bytes())
         else:
@@ -596,7 +679,8 @@ class DAG(collections.abc.Mapping):
         """Plot the DAG
         """
         if output == 'embed':
-            _, path = tempfile.mkstemp(suffix='.png')
+            fd, path = tempfile.mkstemp(suffix='.png')
+            os.close(fd)
         else:
             path = str(output)
 
